{
	open Parser
	open Error

	let keyword_tbl=Hashtbl.create 31
	let _ = List.iter
			(function (keyword, tok) -> Hashtbl.add keyword_tbl keyword tok)
			[("print", PRINT);
			("if", IF);
			("then", THEN);
			("else", ELSE)]

	let s2int s = if s="" then raise (Lex_err ("illegal number token", Error.get_ln()))
				else if ('~' = String.get s 0) then
				- (int_of_string(String.sub s 1 ((String.length s)-1)))
				else int_of_string s
}
let blank=[' ' '\t']+
let id=['a'-'z' 'A'-'Z'](['a'-'z' 'A'-'Z' '\'' '0'-'9' '_'])*
let number=['0'-'9']+|'~'['0'-'9']+

rule start =
 parse	blank	{ start lexbuf }
	| 	"\r\n"	{ incr_ln(); start lexbuf }
	|	'\n'	{ incr_ln(); start lexbuf }
	|	number { INT (s2int(Lexing.lexeme lexbuf)) }
	|	id	{ let id=Lexing.lexeme lexbuf in try Hashtbl.find keyword_tbl id with _ -> ID id }
	|	eof	{ EOF }
	|	";"	{ SEMI }
	|	":="	{ ASSIGN }
	|	"+"	{ PLUS }
	|	"-" { MINUS }
	|	"*" { TIMES }
	|	"/" { DIV }
	|	","	{ COMMA }
	|	"(" { LPAREN }
	|	")" { RPAREN }
	|	_	{ raise (Lex_err ("illegal token "^(Lexing.lexeme lexbuf), get_ln())) }
